/*
 * Copyright 2019 WeBank
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.webank.wedatasphere.linkis.enginemanager.cache

import java.util

import com.webank.wedatasphere.linkis.enginemanager.conf.EngineManagerConfiguration.{CONSOLE_CONFIG_SPRING_APPLICATION_NAME, ENGINE_SPRING_APPLICATION_NAME}
import com.webank.wedatasphere.linkis.protocol.CacheableProtocol
import com.webank.wedatasphere.linkis.protocol.config.{RequestQueryAppConfig, RequestQueryGlobalConfig, ResponseQueryConfig}
import com.webank.wedatasphere.linkis.protocol.engine.RequestEngine
import com.webank.wedatasphere.linkis.protocol.utils.ProtocolUtils
import com.webank.wedatasphere.linkis.rpc.RPCMapCache


/**
  * Created by johnnwang on 2018/11/4.
  */
object ConfigurationMapCache {
  val globalMapCache = new RPCMapCache[RequestEngine, String, String](
    CONSOLE_CONFIG_SPRING_APPLICATION_NAME.getValue)  {
    override protected def createRequest(key: RequestEngine): CacheableProtocol = RequestQueryGlobalConfig(key.user)

    override protected def createMap(any: Any): util.Map[String, String] = any match {
      case response: ResponseQueryConfig => response.getKeyAndValue
    }
  }

  val engineMapCache = new RPCMapCache[RequestEngine, String, String](
    CONSOLE_CONFIG_SPRING_APPLICATION_NAME.getValue)  {
    override protected def createRequest(key: RequestEngine): CacheableProtocol =
      RequestQueryAppConfig(key.user, key.creator, ProtocolUtils.getAppName(ENGINE_SPRING_APPLICATION_NAME.getValue).get)

    override protected def createMap(any: Any): util.Map[String, String] = any match {
      case response: ResponseQueryConfig => response.getKeyAndValue
    }
  }
}
